@@ -308,6 +308,68 @@ indicator_construct (XfcePanelPlugin *plugin)
 }
 
 
+static gboolean
+entry_scrolled (GtkWidget *menuitem, GdkEventScroll *event, gpointer data)
+{
+  IndicatorObject *io = g_object_get_data (G_OBJECT (menuitem), "indicator-custom-object-data");
+  IndicatorObjectEntry *entry = g_object_get_data (G_OBJECT (menuitem), "indicator-custom-entry-data");
+
+  g_return_val_if_fail(INDICATOR_IS_OBJECT(io), FALSE);
+
+  g_signal_emit_by_name (io, "scroll", 1, event->direction);
+  g_signal_emit_by_name (io, "scroll-entry", entry, 1, event->direction);
+
+  return FALSE;
+}
+
+
+static void
+entry_added (IndicatorObject * io, IndicatorObjectEntry * entry, gpointer user_data)
+{
+  GtkWidget * menuitem = gtk_menu_item_new();
+  GtkWidget * hbox = gtk_hbox_new(FALSE, 3);
+
+  g_signal_connect(G_OBJECT(menuitem), "scroll-event", G_CALLBACK(entry_scrolled), entry);
+
+  if (entry->image != NULL)
+    gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(entry->image), FALSE, FALSE, 0);
+
+  if (entry->label != NULL)
+    gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(entry->label), FALSE, FALSE, 0);
+
+  gtk_container_add(GTK_CONTAINER(menuitem), hbox);
+  gtk_widget_show(hbox);
+
+  if (entry->menu != NULL)
+    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), GTK_WIDGET(entry->menu));
+
+  gtk_menu_shell_append(GTK_MENU_SHELL(user_data), menuitem);
+  gtk_widget_show(menuitem);
+
+  g_object_set_data(G_OBJECT(menuitem), "indicator-custom-object-data", io);
+  g_object_set_data(G_OBJECT(menuitem), "indicator-custom-entry-data", entry);
+}
+
+
+static void
+entry_removed_cb (GtkWidget * widget, gpointer userdata)
+{
+  gpointer data = g_object_get_data(G_OBJECT(widget), "indicator-custom-entry-data");
+
+  if (data != userdata)
+    return;
+
+  gtk_widget_destroy(widget);
+}
+
+
+static void
+entry_removed (IndicatorObject * io, IndicatorObjectEntry * entry, gpointer user_data)
+{
+  gtk_container_foreach(GTK_CONTAINER(user_data), entry_removed_cb, entry);
+}
+
+
 static gboolean
 load_module (const gchar * name, GtkWidget * menu)
 {
@@ -323,29 +385,17 @@ load_module (const gchar * name, GtkWidget * menu)
 	IndicatorObject * io = indicator_object_new_from_file(fullpath);
 	g_free(fullpath);
 
+    g_signal_connect(G_OBJECT(io), INDICATOR_OBJECT_SIGNAL_ENTRY_ADDED,
+                     G_CALLBACK(entry_added), menu);
+    g_signal_connect(G_OBJECT(io), INDICATOR_OBJECT_SIGNAL_ENTRY_REMOVED,
+                     G_CALLBACK(entry_removed), menu);
+
 	GList * entries = indicator_object_get_entries(io);
 	GList * entry = NULL;
 
 	for (entry = entries; entry != NULL; entry = g_list_next(entry)) {
 		IndicatorObjectEntry * entrydata = (IndicatorObjectEntry *)entry->data;
-
-		GtkWidget * menuitem = gtk_menu_item_new();
-		GtkWidget * hbox = gtk_hbox_new(FALSE, 3);
-		if (entrydata->image != NULL) {
-			gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(entrydata->image), FALSE, FALSE, 0);
-		}
-		if (entrydata->label != NULL) {
-			gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(entrydata->label), FALSE, FALSE, 0);
-		}
-		gtk_container_add(GTK_CONTAINER(menuitem), hbox);
-		gtk_widget_show(hbox);
-
-		if (entrydata->menu != NULL) {
-			gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), GTK_WIDGET(entrydata->menu));
-		}
-
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-		gtk_widget_show(menuitem);
+		entry_added(io, entrydata, menu);
 	}
 
 	g_list_free(entries);
